/*********************************************************
 * JUnit Extensions
 * Version 1.6.0
 *
 *
 * (C)opyright 2001-2003 Daedalos Consulting GmbH, Witten
 *
 * Author: Jens Uwe Pipka, Daedalos Consulting GmbH
 *      mailto:jens-uwe.pipka@daedalos.com
 *      http://www.daedalos.com/DE/djux
 *
 *********************************************************/
package com.daedalos.dbcheck.test;

import junit.framework.TestCase;

import com.daedalos.dbcheck.DatabaseChecker;
/**
 * Test class for corresponding business class.
 *
 * @author Manfred Duchrow, Jens Uwe Pipka
 * @version 1.0.1
 */
abstract public class CommonDatabaseCheckerTest extends TestCase
{
  /**
   * Initialize the new instance with default values.
   */
  public CommonDatabaseCheckerTest( String name )
  {
    super( name ) ;
  } // CommonDatabaseCheckerTest()

  abstract protected DatabaseChecker dbChecker() ;

  public void test1GetIntColumn()
    throws Exception
  {
    int value  = -1 ;
    value = dbChecker().getIntColumn( "BOOKS", "read_count", "isbn='0-399-12967-7'" );
    this.assertEquals( 1, value );
  } // test1GetIntColumn()

  public void test1GetStringColumn()
    throws Exception
  {
    String value  = null ;
    value = dbChecker().getStringColumn( "BOOKS", "main_author", "isbn = '0-201-13688-0'" );
    this.assertEquals( "GOLDBERG", value );
  } // test1GetStringColumn()

  public void test1RowCount()
    throws Exception
  {
    long rows   = 0 ;

    rows = dbChecker().rowCount( "CATEGORY" ) ;
    this.assertEquals( 42L, rows );
  } // test1RowCount()
  // -------------------------------------------------------------------------

  public void test1RowExists()
    throws Exception
  {
    this.assertTrue( dbChecker().rowExists( "AUTHORS", "id = 'ASPRIN'" ) );
  } // test1RowExists()
  // -------------------------------------------------------------------------

  public void test1RowsExist()
    throws Exception
  {
    this.assertTrue( dbChecker().multipleRowsExist( "BOOKS", "publisher = 'PENGUIN'" ) );
  } // test1RowsExist()
  // -------------------------------------------------------------------------

  public void test1TableExists()
    throws Exception
  {
    this.assertTrue( dbChecker().tableExists("BOOKS") );
  } // test1TableExists()
  // -------------------------------------------------------------------------

  public void test2RowCount()
    throws Exception
  {
    long rows   = 0 ;

    rows = dbChecker().rowCount( "BOOKS", "MAIN_AUTHOR='GEMMELL'" ) ;
    this.assertEquals( 6L, rows );
  } // test2RowCount()
  // -------------------------------------------------------------------------

  public void test2RowExists()
    throws Exception
  {
    this.assertTrue( ! dbChecker().rowExists( "AUTHORS", "id = 'XYZ'" ) );
  } // test2RowExists()
  // -------------------------------------------------------------------------

  public void test2RowsExist()
    throws Exception
  {
    this.assertTrue( ! dbChecker().multipleRowsExist( "BOOKS", "title = 'CAXCXAC'" ) );
  } // test2RowsExist()
  // -------------------------------------------------------------------------

  public void test2TableExists()
    throws Exception
  {
    this.assertTrue( ! dbChecker().tableExists("UNKNOWN") );
  } // test2TableExists()
  // -------------------------------------------------------------------------

  public void test3RowCount()
    throws Exception
  {
    long rows   = 0 ;

    rows = dbChecker().rowCount( "BOOKS", "MAIN_AUTHOR='BSHZEZE'" ) ;
    this.assertEquals( 0L, rows );
  } // test3RowCount()
}
