/*********************************************************
 * JUnit Extensions
 * Version 1.6.0
 *
 *
 * (C)opyright 2001-2003 Daedalos Consulting GmbH, Witten
 *
 * Author: Jens Uwe Pipka, Daedalos Consulting GmbH
 *      mailto:jens-uwe.pipka@daedalos.com
 *      http://www.daedalos.com/DE/djux
 *
 *********************************************************/
package com.daedalos.dbcheck.test;

import java.sql.Connection;
import java.sql.Driver;
import java.sql.DriverManager;
import java.sql.SQLException;

import junit.framework.Test;
import junit.framework.TestSuite;

import com.daedalos.dbcheck.DatabaseChecker;

/**
 * Test class for corresponding business class.
 *
 * @author Manfred Duchrow, Jens Uwe Pipka
 * @version 1.0, 1.0.1
 */
public class DatabaseCheckerTest2 extends CommonDatabaseCheckerTest
{
  private static final String DATABASE_NAME 			= "jdbc:odbc:UnitTestDB" ;
  private static final boolean DRIVER_REGISTERED  = registerDatabaseDriver() ;

  private DatabaseChecker databaseChecker = null;
  private Connection connection = null ;

  /**
   * Initialize the new instance with default values.
   */
  public DatabaseCheckerTest2( String name )
  {
    super( name ) ;
  } // DatabaseCheckerTest2()

  protected Connection createConnection()
	throws SQLException
  {
	return DriverManager.getConnection( DATABASE_NAME ) ;
  } // createConnection()

  protected DatabaseChecker dbChecker()
  {
    return this.getDatabaseChecker() ;
  } // dbChecker()

  protected Connection getConnection() { return connection ; }

  protected DatabaseChecker getDatabaseChecker() { return databaseChecker ; }

  protected static boolean registerDatabaseDriver()
  {
		Driver driver 			= null ;
		boolean registered 	= false ;

		try
		{
			driver = (Driver)Class.forName("sun.jdbc.odbc.JdbcOdbcDriver").newInstance() ;
			DriverManager.registerDriver( driver ) ;
			registered = true ;
		}
		catch (Exception e)
		{
			System.err.println("Exception obtaining connection:" + e );
		}
		return registered ;
  } // registerDatabaseDriver()
  
  protected void setConnection( Connection newValue ) { connection = newValue ; }
  
  protected void setDatabaseChecker( DatabaseChecker newValue ) { databaseChecker = newValue ; }

  // TEST METHODS
  // =========================================================================
  public void setUp()
  {
    if ( DRIVER_REGISTERED )
    {
      try
      {
        this.setConnection( this.createConnection() ) ;
        this.setDatabaseChecker( new DatabaseChecker( this.getConnection() ) ) ;
      }
      catch ( SQLException ex  )
      {
        System.err.println( ex.getMessage() ) ;
        ex.printStackTrace() ;
      }
    }
  } // setUp

   public static Test suite()
  {
		return new TestSuite( DatabaseCheckerTest2.class ) ;
  } // suite()

  public void tearDown()
  {
    if ( this.getConnection() != null )
    {
      try
      {
        this.getConnection().close() ;
      }
      catch ( Exception ex ) {}
      this.setConnection( null ) ;
      this.setDatabaseChecker( null ) ;
    }
  } // tearDown()
} // class DatabaseCheckerTest2
