/*********************************************************
 * JUnit Extensions
 * Version 1.6.0
 *
 *
 * (C)opyright 2001-2003 Daedalos Consulting GmbH, Witten
 *
 * Author: Jens Uwe Pipka, Daedalos Consulting GmbH
 *      mailto:jens-uwe.pipka@daedalos.com
 *      http://www.daedalos.com/DE/djux
 *
 *********************************************************/
package com.daedalos.junit.gui;

import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.io.IOException;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JTable;
import javax.swing.SwingUtilities;
import javax.swing.table.TableColumn;

import com.daedalos.junit.TestResourceException;
import com.daedalos.junit.TestResourceFactory;

public class TRBrowser extends JFrame implements Observer {
	private TRBrowserModel trBrowserModel;
	private TRBrowserTableModel trBrowserTableModel;

	private JMenuItem stopAllMenuItem;
	private JMenuItem closeMenuItem;
	private JMenuBar trBrowserMenuBar;
	private JScrollPane trBrowserScrollPane;
	private JMenuItem stopTestResourcePopupMenuItem;
	private JMenuItem infoDialogMenuItem;
	private JMenu helpMenu;
	private JPopupMenu trBrowserPopupMenu;
	private JMenuItem startTestResourcePopupMenuItem;
	private JMenu configurationMenu;
	private JCheckBoxMenuItem globalTestResourcesMenuItem;
	private JMenu resourcesMenu;
	private JMenuItem removeTestResourceMenuItem;
	private JCheckBoxMenuItem saveWindowsPropertiesMenuItem;
	private JMenuItem startMenuItem;
	private JCheckBoxMenuItem autostartTestResourcesMenuItem;
	private JMenuItem addTestResourceMenuItem;
	private JMenuItem startAllMenuItem;
	private JSeparator jSeparator3;
	private JSeparator jSeparator2;
	private JSeparator jSeparator1;
	private JMenuItem stopMenuItem;
	private JMenu fileMenu;
	private JMenuItem removeTestResourcePopupMenuItem;
	private JTable trBrowserTable;

	/** Creates new form TRBrowser */
	public TRBrowser() {
		trBrowserModel = new TRBrowserModel();
		trBrowserModel.addObserver(this);
		this.initComponents();
		TRBrowserTableCellRenderer trBrowserCellRenderer =
			new TRBrowserTableCellRenderer();
		TableColumn iconColumn = trBrowserTable.getColumnModel().getColumn(0);
		iconColumn.setCellRenderer(trBrowserCellRenderer);
		if (trBrowserModel.getSaveWindowProperties()) {
			this.setSize(trBrowserModel.getWidth(), trBrowserModel.getHeight());
			this.setLocation(trBrowserModel.getPosX(), trBrowserModel.getPosY());
		}
	}

	/** This method is called from within the constructor to
	 * initialize the form.
	 */
	private void initComponents() {
		trBrowserPopupMenu = new JPopupMenu();
		startTestResourcePopupMenuItem = new JMenuItem();
		stopTestResourcePopupMenuItem = new JMenuItem();
		jSeparator3 = new JSeparator();
		removeTestResourcePopupMenuItem = new JMenuItem();
		trBrowserScrollPane = new JScrollPane();
		trBrowserTable = new JTable();
		trBrowserMenuBar = new JMenuBar();
		fileMenu = new JMenu();
		closeMenuItem = new JMenuItem();
		resourcesMenu = new JMenu();
		startMenuItem = new JMenuItem();
		stopMenuItem = new JMenuItem();
		jSeparator1 = new JSeparator();
		startAllMenuItem = new JMenuItem();
		stopAllMenuItem = new JMenuItem();
		configurationMenu = new JMenu();
		addTestResourceMenuItem = new JMenuItem();
		removeTestResourceMenuItem = new JMenuItem();
		jSeparator2 = new JSeparator();
		globalTestResourcesMenuItem = new JCheckBoxMenuItem();
		autostartTestResourcesMenuItem = new JCheckBoxMenuItem();
		saveWindowsPropertiesMenuItem = new JCheckBoxMenuItem();
		helpMenu = new JMenu();
		infoDialogMenuItem = new JMenuItem();
		TestResourceFactory testResourceFactory = TestResourceFactory.current();

		startTestResourcePopupMenuItem.setText("Start Test Resource");
		startTestResourcePopupMenuItem
			.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				startTestResourcePopupMenuItemActionPerformed(evt);
			}
		});

		trBrowserPopupMenu.add(startTestResourcePopupMenuItem);
		stopTestResourcePopupMenuItem.setText("Stop Test Resource");
		stopTestResourcePopupMenuItem
			.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				stopTestResourcePopupMenuItemActionPerformed(evt);
			}
		});

		trBrowserPopupMenu.add(stopTestResourcePopupMenuItem);
		trBrowserPopupMenu.add(jSeparator3);
		removeTestResourcePopupMenuItem.setText("Remove Test Resource");
		removeTestResourcePopupMenuItem
			.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				removeTestResourcePopupMenuItemActionPerformed(evt);
			}
		});

		trBrowserPopupMenu.add(removeTestResourcePopupMenuItem);

		getContentPane().setLayout(new java.awt.GridLayout(1, 0));

		setTitle("Test Resource Browser");
		addWindowListener(new java.awt.event.WindowAdapter() {
			public void windowClosing(java.awt.event.WindowEvent evt) {
				updateTRBrowserConfig();
				updateTRFactoryConfig();
			}
		});

		trBrowserTable.setModel(this.getTableModel());
		trBrowserTable.addMouseListener(new java.awt.event.MouseAdapter() {
			public void mousePressed(java.awt.event.MouseEvent evt) {
				trBrowserTableMousePressed(evt);
			}
		});

		trBrowserScrollPane.setViewportView(trBrowserTable);

		getContentPane().add(trBrowserScrollPane);

		fileMenu.setText("File");
		closeMenuItem.setText("Close");
		closeMenuItem.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				closeMenuItemActionPerformed(evt);
			}
		});

		fileMenu.add(closeMenuItem);
		trBrowserMenuBar.add(fileMenu);

		resourcesMenu.setText("Resources");
		startMenuItem.setText("Start");
		startMenuItem.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				startMenuItemActionPerformed(evt);
			}
		});
		resourcesMenu.add(startMenuItem);
		stopMenuItem.setText("Stop");
		stopMenuItem.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				stopMenuItemActionPerformed(evt);
			}
		});
		resourcesMenu.add(stopMenuItem);
		resourcesMenu.add(jSeparator1);
		startAllMenuItem.setText("Start all");
		startAllMenuItem.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				startAllMenuItemActionPerformed(evt);
			}
		});
		resourcesMenu.add(startAllMenuItem);

		stopAllMenuItem.setText("Stop all");
		stopAllMenuItem.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				stopAllMenuItemActionPerformed(evt);
			}
		});
		resourcesMenu.add(stopAllMenuItem);
		trBrowserMenuBar.add(resourcesMenu);

		configurationMenu.setText("Configuration");
		addTestResourceMenuItem.setText("Add Test Resource");
		addTestResourceMenuItem.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				addTestResourceMenuItemActionPerformed(evt);
			}
		});
		configurationMenu.add(addTestResourceMenuItem);
		removeTestResourceMenuItem.setText("Remove Test Resource");
		removeTestResourceMenuItem
			.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				removeTestResourceMenuItemActionPerformed(evt);
			}
		});

		configurationMenu.add(removeTestResourceMenuItem);
		configurationMenu.add(jSeparator2);
		globalTestResourcesMenuItem.setSelected(
			testResourceFactory.getGlobalTestResources());
		globalTestResourcesMenuItem.setText("Global Test Resources");
		globalTestResourcesMenuItem
			.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				globalTestResourcesMenuItemActionPerformed(evt);
			}
		});

		configurationMenu.add(globalTestResourcesMenuItem);
		autostartTestResourcesMenuItem.setSelected(
			testResourceFactory.getStartTestResources());
		autostartTestResourcesMenuItem.setText("Autostart Test Resources");
		autostartTestResourcesMenuItem
			.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				autostartTestResourcesMenuItemActionPerformed(evt);
			}
		});

		configurationMenu.add(autostartTestResourcesMenuItem);
		saveWindowsPropertiesMenuItem.setSelected(
			trBrowserModel.getSaveWindowProperties());
		saveWindowsPropertiesMenuItem.setText("Save Window Properties");
		saveWindowsPropertiesMenuItem
			.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				saveWindowsPropertiesMenuItemActionPerformed(evt);
			}
		});

		configurationMenu.add(saveWindowsPropertiesMenuItem);
		trBrowserMenuBar.add(configurationMenu);
		helpMenu.setText("Help");

		infoDialogMenuItem.setText("Info");
		infoDialogMenuItem.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent evt) {
				infoDialogMenuItemActionPerformed(evt);
			}
		});

		helpMenu.add(infoDialogMenuItem);
		trBrowserMenuBar.add(helpMenu);
		setJMenuBar(trBrowserMenuBar);

		pack();
		java.awt.Dimension screenSize =
			java.awt.Toolkit.getDefaultToolkit().getScreenSize();
		setSize(new java.awt.Dimension(400, 300));
		setLocation((screenSize.width - 400) / 2, (screenSize.height - 300) / 2);
	}

	private void removeTestResourcePopupMenuItemActionPerformed(ActionEvent evt) {
		this.removePopupMenuItemActionPerformed(evt);
	}

	private void stopTestResourcePopupMenuItemActionPerformed(ActionEvent evt) {
		this.stopMenuItemActionPerformed(evt);
	}

	private void startTestResourcePopupMenuItemActionPerformed(ActionEvent evt) {
		this.startMenuItemActionPerformed(evt);
	}

	private void infoDialogMenuItemActionPerformed(ActionEvent evt) {
		JOptionPane infoOptionPane =
			new JOptionPane(
				"Test Resource Browser\n" + "jens-uwe.pipka@daedalos.com",
				JOptionPane.INFORMATION_MESSAGE,
				JOptionPane.DEFAULT_OPTION,
				null,
				null);
		JDialog infoDialog = infoOptionPane.createDialog(this, "Info");
		infoDialog.show();
	}

	private void saveWindowsPropertiesMenuItemActionPerformed(ActionEvent evt) {
		trBrowserModel.setSaveWindowProperties(
			saveWindowsPropertiesMenuItem.getState());
	}

	private void autostartTestResourcesMenuItemActionPerformed(ActionEvent evt) {
		TestResourceFactory testResourceFactory = TestResourceFactory.current();
		testResourceFactory.setStartTestResources(
			autostartTestResourcesMenuItem.getState());
	}

	private void globalTestResourcesMenuItemActionPerformed(ActionEvent evt) {
		TestResourceFactory testResourceFactory = TestResourceFactory.current();
		testResourceFactory.setGlobalTestResources(
			globalTestResourcesMenuItem.getState());
	}

	private void removeTestResourceMenuItemActionPerformed(ActionEvent evt) {
		int[] selectedRows = trBrowserTable.getSelectedRows();
		for (int i = 0; i < selectedRows.length; i++) {
			trBrowserModel.removeTestResourceAt(selectedRows[i]);
		}
	}

	private void addTestResourceMenuItemActionPerformed(ActionEvent evt) {
		TRRegisterTestResource trRegisterDialog =
			new TRRegisterTestResource(new JFrame(), true);
		trRegisterDialog.show();
		String trDescription = trRegisterDialog.getTRDescription();
		String trClassname = trRegisterDialog.getTRClassname();
		try {
			trBrowserModel.registerNewTestResource(trDescription, trClassname);
		} catch (TestResourceException e) {
			JOptionPane infoOptionPane =
				new JOptionPane(
					"Test Resource " + trClassname + " does not exist!",
					JOptionPane.INFORMATION_MESSAGE,
					JOptionPane.DEFAULT_OPTION,
					null,
					null);
			JDialog infoDialog = infoOptionPane.createDialog(this, "Info");
			infoDialog.show();
		}
	}

	private void trBrowserTableMousePressed(MouseEvent evt) {
		if (SwingUtilities.isRightMouseButton(evt)) {
			trBrowserPopupMenu.show(evt.getComponent(), evt.getX(), evt.getY());
		}
	}

	private void removePopupMenuItemActionPerformed(ActionEvent evt) {
		this.removeTestResourceMenuItemActionPerformed(evt);
	}

	private void stopMenuItemActionPerformed(ActionEvent evt) {
		int[] selectedRows = trBrowserTable.getSelectedRows();
		for (int i = 0; i < selectedRows.length; i++) {
			trBrowserModel.tearDownTestResourceAt(selectedRows[i]);
		}
	}

	private void startAllMenuItemActionPerformed(ActionEvent evt) {
		trBrowserModel.setUpAllTestResources();
	}

	private void stopAllMenuItemActionPerformed(ActionEvent evt) {
		trBrowserModel.tearDownAllTestResources();
	}

	private void startMenuItemActionPerformed(ActionEvent evt) {
		boolean success = true;
		int[] selectedRows = trBrowserTable.getSelectedRows();
		for (int i = 0; i < selectedRows.length; i++) {
			if (!trBrowserModel.setUpTestResourceAt(selectedRows[i])) {
				success = false;
			}
		}
		if (!success) {	
			JOptionPane infoOptionPane =
				new JOptionPane(
					"Not all Test Resources could be started!",
					JOptionPane.INFORMATION_MESSAGE,
					JOptionPane.DEFAULT_OPTION,
					null,
					null);
			JDialog infoDialog = infoOptionPane.createDialog(this, "Info");
			infoDialog.show();
		}

	}

	private void updateTRBrowserConfig() {
		if (trBrowserModel.getSaveWindowProperties()) {
			trBrowserModel.setPosX(new Double(getLocation().getX()).intValue());
			trBrowserModel.setPosY(new Double(getLocation().getY()).intValue());
			trBrowserModel.setWidth(getWidth());
			trBrowserModel.setHeight(getHeight());
		}
		TestResourceFactory testResourceFactory = TestResourceFactory.current();
		testResourceFactory.setGlobalTestResources(
			globalTestResourcesMenuItem.getState());
		trBrowserModel.setSaveWindowProperties(
			saveWindowsPropertiesMenuItem.getState());
		trBrowserModel.writeTRBrowserConfigFile();
	}

	private void updateTRFactoryConfig() {
		try {
			TestResourceFactory testResourceFactory = TestResourceFactory.current();
			testResourceFactory.saveTestResourceConfigFile();
			testResourceFactory.saveUserConfigFile();
		} catch (IOException e) {
			throw new TestResourceException("TestResourceBrowser configuration file could not be saved.");
		}
	}

	private void closeMenuItemActionPerformed(ActionEvent evt) {
		this.updateTRBrowserConfig();
		this.updateTRFactoryConfig();
		this.dispose();
	}

	public TRBrowserTableModel getTableModel() {
		if (trBrowserTableModel == null) {
			trBrowserTableModel = new TRBrowserTableModel(trBrowserModel);
			trBrowserModel.addObserver(this.trBrowserTableModel);
		}
		return trBrowserTableModel;
	}

	public void update(Observable o, Object arg) {
		this.paint(this.getGraphics());
	}

	/**
	* @param args the command line arguments
	*/
	public static void main(String args[]) {
		TRBrowser aTRBrowser;
		aTRBrowser = new TRBrowser();
		aTRBrowser.show();
	}

}