/*********************************************************
 * JUnit Extensions
 * Version 1.6.0
 *
 *
 * (C)opyright 2001-2003 Daedalos Consulting GmbH, Witten
 *
 * Author: Jens Uwe Pipka, Daedalos Consulting GmbH
 *      mailto:jens-uwe.pipka@daedalos.com
 *      http://www.daedalos.com/DE/djux
 *
 *********************************************************/
package com.daedalos.junit.gui;

import java.util.Observable;
import java.util.Observer;

import javax.swing.Icon;

import junit.swingui.TestRunner;

import com.daedalos.junit.TestResource;

class TRBrowserTableModel
	extends javax.swing.table.AbstractTableModel
	implements Observer {

	protected String[] columnNames;
	protected Class[] columnClasses;
	protected TRBrowserModel testResourcesBrowserModel;
	protected int columns;
	protected Icon okIcon;
	protected Icon errorIcon;

	public TRBrowserTableModel(TRBrowserModel aTRBModel) {
		super();
		this.testResourcesBrowserModel = aTRBModel;
		this.initializeColumns();
		this.loadIcons();
	}

	protected java.util.Vector entries() {
		return testResourcesBrowserModel.getTestResources();
	}

	public Class getColumnClass(int col) {
		return columnClasses[col];
	}

	public int getColumnCount() {
		return columns;
	}

	public String getColumnName(int col) {
		return columnNames[col];
	}

	public int getRowCount() {
		return entries().size();
	}

	public Object getValueAt(int row, int col) {
		TestResource entry = (TestResource) entries().elementAt(row);
		switch (col) {
			case 0 :
				return this.testResourceIsAvailable(entry);
			case 1 :
				return entry.getName();
			case 2 :
				return entry.getDescription();
			case 3 :
				return entry.getState();
			default :
				return null;
		}
	}

	public void initializeColumns() {
		columnNames = new String[] { "", "Name", "Description", "State" };
		columnClasses =
			new Class[] { Icon.class, String.class, String.class, String.class };
		columns = 4;
	}

	protected void loadIcons() {
		//Use the standard JUnit Icons
		okIcon =
			TestRunner.getIconResource(new TestRunner().getClass(), "icons/ok.gif");
		errorIcon =
			TestRunner.getIconResource(new TestRunner().getClass(), "icons/error.gif");
	}

	public Icon testResourceIsAvailable(TestResource aTestResource) {
		if (aTestResource.isStarted())
			return okIcon;
		else
			return errorIcon;
	}

	public void update(Observable source, Object object) {
		this.fireTableDataChanged();
	}

}