/*********************************************************
 * Daedalos Class Locator 
 * Version 1.6.0
 *
 *
 * (C)opyright 2003 Daedalos Consulting GmbH, Witten
 *
 * Author: Jens Uwe Pipka, Daedalos Consulting GmbH
 *      mailto:jens-uwe.pipka@daedalos.com
 *      http://www.daedalos.com/
 *
 *********************************************************/
package com.daedalos.util;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.StringTokenizer;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

public class ClassLocator {
	private String classname;
	
	public static void main(String[] args) {
		ClassLocator runner = new ClassLocator();
		runner.printHeader();
		
		if (args.length <= 0) {
			runner.printUsage();
		} else {
			String result = runner.locateJavaClass(args[0]);
			if (result == null) {
				System.out.println("Not found!");
			} else {
				System.out.println("Found in " + result);
				if (!runner.isLoadable()) {
					System.out.println("Warning: Class ist not loadable!");
				}
			}
				
		}
	}
	
	public String locateJavaClass(String classname) {
		this.classname = classname;
		String classpath = System.getProperty("java.class.path");
		ArrayList list = this.scanPath(classpath);
		String result = this.scanEntries(list);
		if ((result == null) && (this.isLoadable()))
			result = "Java Environment";
		return result;
	}
	
	private boolean isLoadable() {
		try {
		 	Class aClass = Class.forName(classname);
		 	return aClass != null;
		} catch (NoClassDefFoundError e) {
			return false;
		} catch (Exception e) {
			return false;
		}
	}
			
	private ArrayList scanPath(String classPath) {
		String separator= System.getProperty("path.separator");
		ArrayList pathEntries = new ArrayList();
		StringTokenizer st = new StringTokenizer(classPath, separator);
		while (st.hasMoreTokens()) {
			pathEntries.add(st.nextToken());
		}
		return pathEntries;
	}	
	
	private String scanEntries(ArrayList pathEntries) {
		Iterator iterator = pathEntries.iterator();
		while (iterator.hasNext()) {
			String entry= (String) iterator.next();
			if (entry.endsWith(".jar") || entry.endsWith(".zip")) {
				String result = this.analyzeJar(entry);
				if (result != null)
					return result;
			} else {
				String result = this.analyzeDir(entry, "");
				if (result != null)
					return result;
			}
		}
		return null;
	}
	
	private String analyzeJar(String path) {
		ZipFile zipFile = null;
		File jar = new File(path);
		if (!jar.exists())
			return null;
		try {
			zipFile= new ZipFile(jar);
		} catch(IOException io) {
			return null;
		}
		Enumeration enu = zipFile.entries();
		while (enu.hasMoreElements()) {
			ZipEntry entry = (ZipEntry)enu.nextElement();
			if (!entry.isDirectory()) {
				String filename = entry.getName().replace('/', '.');				
				if (filename.equals(classname + ".class")) {
					return path;
				}
			}
		}
		return null;
	}
			
	private String analyzeDir(String path, String packageName) {
		File dir = new File(path);		
		if (dir.isDirectory()) {			
			File[] subDirs = dir.listFiles();
			for (int i=0; i<subDirs.length; i++) {
				if (subDirs[i].isDirectory()) {
					String prefix = packageName;
					if (packageName.length()>0) prefix = prefix + ".";
					prefix = prefix + subDirs[i].getName();
					String result = this.analyzeDir(subDirs[i].getAbsolutePath(), prefix);
					if (result != null) {
						return result;
					}
				} else {
					String filename = packageName + "." + subDirs[i].getName();									
					if (filename.equals(classname+".class")) {
						return subDirs[i].getAbsolutePath();
					}
				}
			}
		}
		return null;
	}	
	
	protected void printHeader() {
		System.out.println("Java Class Locator 1.0");	
		System.out.println("(C)opyright 2003 Daedalos Consulting GmbH, Jens Uwe Pipka");		
		System.out.println();
	}		
	
	protected void printUsage() {
		System.out.println("Usage: ClassLocator FullyQualifiedClassname");
	}
	
}

